//A unit providing procedures to re/store any window or form
//position & visibility data from/into the registry or a stream.

//Copyright(c) 2001-4 by Tomas Koutny.
//This file is freeware. Version 3.0

//------------------------------------------------------------------------------
//WARRANTIES AND DISCLAIMER:
//
//EXCEPT AS EXPRESSLY PROVIDED OTHERWISE IN A WRITTEN AGREEMENT BETWEEN
//Tomas Koutny ("LICENSOR") AND YOU, THE LICENSED WORKS ARE NOW PROVIDED "AS IS"
//WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT
//LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//FOR A PARTICULAR PURPOSE, OR THE WARRANTY OF NON-INFRINGEMENT.

//IN NO EVENT SHALL LICENSOR OR ITS SUPPLIERS BE LIABLE TO YOU OR ANY THIRD
//PARTY FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
//KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING, WITHOUT LIMITATION, THOSE
//RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT LICENSOR HAD BEEN
//ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, AND ON ANY THEORY OF LIABILITY,
//ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE LICENSED WORKS. SOME
//JURISDICTIONS PROHIBIT THE EXCLUSION OR LIMITATION OF LIABILITY FOR
//CONSEQUENTIAL OR INCIDENTAL DAMAGES, SO THE ABOVE LIMITATIONS MAY NOT APPLY TO
//YOU. THESE LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL
//PURPOSE OF ANY LIMITED REMEDY.
//------------------------------------------------------------------------------

//See www.rawos.com for more software and resources.

(*
  Version history:
  ================

  3.0
     - Added support for TToolbar saving
  2.0 Revision 2
     - minor enhancement done for a VCL form restoring
     - fixed a very occasionally bug occuring with the VCL
  2.0 Revision 1
     - StoreWindowState & RestoreWindowState were renamed
       to StoreFormState & RestoreFormState to remain 'VCL use only'
     - Added StoreWindowState & RestoreWindowState working with WinAPI.
     - RestoreWindowState is capable to recognize and ignore TFormState data.
     - Added support for streams for both 'VCL use only' & WinAPI
  1.0  Revision 1
     - Released for the first time
*)
unit WindowSaver;

interface

uses Classes, Forms, Windows, Registry, Controls, ComCtrls;

type
  TToolbarPlacement = packed record
                        LRDockWidth,
                        TBDockHeight,
                        UndockWidth,
                        UndockHeight:Longint;
                        Orientation:TDockOrientation;
                        ScreenPos:TRect;
                        Visible:WordBool;
                        //-------------------Member that must be set standalone
                        //-------------------to identify Dock Site control
                        DockSiteID:LongInt;
                      end;

//Procedures using WinAPI routines
//To use these routines with Forms, it is strongly recommended to use
//the TForm variants instead of HWND ones since it is not the VCL native way.
procedure StoreWindowState(AWnd:HWND; ARegistry:TRegistry; AValueName:String);   overload;
procedure StoreWindowState(AWnd:TForm; ARegistry:TRegistry; AValueName:String);  overload;
procedure StoreWindowState(AWnd:HWND; AStream:TStream);                          overload;
procedure StoreWindowState(AWnd:TForm; AStream:TStream);                         overload;
procedure RestoreWindowState(AWnd:HWND; ARegistry:TRegistry; AValueName:String); overload;
procedure RestoreWindowState(AWnd:TForm; ARegistry:TRegistry; AValueName:String);overload;
procedure RestoreWindowState(AWnd:HWND; AStream:TStream);                        overload;
procedure RestoreWindowState(AWnd:TForm; AStream:TStream);                       overload;


//Procedures operation the pure VCL way
procedure StoreFormState(AForm:TForm; ARegistry:TRegistry; AValueName:String);   overload;
procedure StoreFormState(AForm:TForm; AStream:TStream);                          overload;
procedure RestoreFormState(AForm:TForm; ARegistry:TRegistry; AValueName:String); overload;
procedure RestoreFormState(AForm:TForm; AStream:TStream);                        overload;


//TToolbar procedures using VCL way
procedure StoreToolbarPlacement(AToolbar:TToolBar;
                                var APlacement:TToolbarPlacement);
procedure DockToolbar(AToolbar:TToolBar;
                      APlacement:TToolbarPlacement; ADockSite:TWinControl);
        //Remember to make all toolbar floating first and to set ADockSite
        //proper height before doing any dock operation
        //Set ADockSite to nil to identify floating

implementation

type
  //To use only VCL methods, we need to declare some record to store form's data
  TFormState = packed record
                  Visible:wordbool;
                  State:longint;        //TWindowState
                  Left,
                  Top,
                  Width,
                  Height:longint;
               end;

  //Because Set/GetWindowPlacement don't handle the visibility, we have
  //to take care about it
  TWndState = packed record
                Placement:TWindowPlacement;
                        //The first member allows the recognization
                        //of any invalid data
                Visible:WordBool;
              end;

//----------------------------------------------------------Pure VCL utilization

function GetFormState(AForm:TForm):TFormState;
//Retrieves all necessary information about the form
begin
  with AForm do
    begin
      result.Visible:=Visible;
      result.State:=integer(WindowState);
      result.Left:=Left;
      result.Top:=Top;
      result.Width:=Width;
      result.Height:=Height;
    end;
end;

procedure SetFormState(AForm:TForm; AData:TFormState);
//Applies all useable information to the form
begin
  with AForm do
    begin
      Visible:=AData.Visible;   //Sometimes it can change size of the form
                                //when called after the setbounds method
      if TWindowState(AData.State)=wsNormal then
        AForm.SetBounds(AData.Left, AData.Top, AData.Width, AData.Height);
      WindowState:=TWindowState(AData.State);
      //Must be last else the maximaze process is not guaranteed to occure        
    end;
end;

procedure StoreFormState(AForm:TForm; ARegistry:TRegistry; AValueName:String);
var temp:TFormState;
begin
  temp:=GetFormState(AForm);
  ARegistry.WriteBinaryData(AValueName, temp, sizeof(temp));
end;

procedure RestoreFormState(AForm:TForm; ARegistry:TRegistry; AValueName:String);
var temp:TFormState;
begin
  with ARegistry do
    if ValueExists(AValueName) then
      begin
        ReadBinaryData(AValueName, temp, sizeof(temp));
        SetFormState(AForm, temp);
      end;
end;

procedure StoreFormState(AForm:TForm; AStream:TStream);
var temp:TFormState;
begin
  temp:=GetFormState(AForm);
  AStream.Write(temp, sizeof(temp));
end;

procedure RestoreFormState(AForm:TForm; AStream:TStream);
var temp:TFormState;
begin
  AStream.Read(temp, sizeof(temp));
  SetFormState(AForm, temp);
end;


//------------------------------------------------------------WinAPI utilization

function GetWndState(AWnd:HWND):TWndState;
begin
  with result do
    begin
      Visible:=IsWindowVisible(AWnd);
      Placement.length:=sizeof(TWindowPlacement);
      GetWindowPlacement(AWnd, @Placement);
    end;
end;

procedure SetWndState(AWnd:HWND; AData:TWndState); overload; 
begin
  with AData do
    begin
      if Placement.Length <> sizeof(TWindowPlacement) then exit;
            //Restore if the record seems to comply with its values
      SetWindowPlacement(AWnd, @Placement);

      if Visible then ShowWindow(AWnd, SW_SHOW)
        else ShowWindow(AWnd, SW_HIDE);
    end;
end;

//Beacause VCL can bravely resist the WM_SHOWWINDOW message, it must be
//declared in this way.
procedure SetWndState(AWnd:TForm; AData:TWndState); overload; 
begin
  with AWnd do
    begin
      SetWndState(Handle, AData);
      Visible:=AData.Visible;
    end;
end;

procedure StoreWindowState(AWnd:HWND; ARegistry:TRegistry; AValueName:String);
var temp:TWndState;
begin
  temp:=GetWndState(AWnd);
  ARegistry.WriteBinaryData(AValueName, temp, sizeof(TWndState));
end;


procedure RestoreWindowState(AWnd:HWND; ARegistry:TRegistry; AValueName:String);
var temp:TWndState;
begin
  with ARegistry do
    if ValueExists(AValueName) then
      begin
        ReadBinaryData(AValueName, temp, sizeof(TWndState));

        SetWndState(AWnd, temp);
      end;
end;

procedure StoreWindowState(AWnd:TForm; ARegistry:TRegistry; AValueName:String);
begin
  StoreWindowState(AWnd.Handle, ARegistry, AValueName);
end;

procedure RestoreWindowState(AWnd:TForm; ARegistry:TRegistry; AValueName:String);
var temp:TWndState;
begin
  with ARegistry do
    if ValueExists(AValueName) then
      begin
        ReadBinaryData(AValueName, temp, sizeof(TWndState));

        SetWndState(AWnd, temp);
      end;
end;

procedure StoreWindowState(AWnd:HWND; AStream:TStream);
var temp:TWndState;
begin
  temp:=GetWndState(AWnd);
  AStream.WriteBuffer(temp, sizeof(TWndState));
end;

procedure RestoreWindowState(AWnd:HWND; AStream:TStream);
var temp:TWndState;
begin
  AStream.ReadBuffer(temp, sizeof(TWndState));
  SetWndState(AWnd, temp);
end;

procedure StoreWindowState(AWnd:TForm; AStream:TStream);
begin
  StoreWindowState(AWnd.Handle, AStream);
end;

procedure RestoreWindowState(AWnd:TForm; AStream:TStream);
var temp:TWndState;
begin
  AStream.ReadBuffer(temp, sizeof(TWndState));
  SetWndState(AWnd, temp);
end;

//---------------------------------------------TToolbar procedures using VCL way

procedure StoreToolbarPlacement(AToolbar:TToolBar;
                                var APlacement:TToolbarPlacement);
begin
  APlacement.LRDockWidth:=AToolbar.LRDockWidth;
  APlacement.TBDockHeight:=AToolbar.TBDockHeight;
  APlacement.UndockWidth:=AToolbar.UndockWidth;
  APlacement.UndockHeight:=AToolbar.UndockHeight;

  APlacement.Orientation:=AToolbar.DockOrientation;

  APlacement.ScreenPos:=AToolbar.BoundsRect;

  if AToolbar.Floating then
    begin
      APlacement.ScreenPos.TopLeft:=AToolbar.ClientToScreen(
                                                  APlacement.ScreenPos.TopLeft);
      APlacement.ScreenPos.BottomRight:=AToolbar.ClientToScreen(
                                              APlacement.ScreenPos.BottomRight);
    end;

  APlacement.Visible:=AToolbar.Visible;
end;

procedure DockToolbar(AToolbar:TToolBar;
                      APlacement:TToolbarPlacement; ADockSite:TWinControl);
begin
  AToolbar.UndockWidth:=APlacement.UndockWidth;
  AToolbar.UndockHeight:=APlacement.UndockHeight;
  AToolbar.LRDockWidth:=APlacement.LRDockWidth;
  AToolbar.TBDockHeight:=APlacement.TBDockHeight;

  AToolbar.DockOrientation:=APlacement.Orientation;

  if ADockSite <> nil then AToolbar.Dock(ADockSite, APlacement.ScreenPos)
    else AToolbar.ManualFloat(APlacement.ScreenPos);

  AToolbar.Visible:=APlacement.Visible;
end;

end.
